package nu.zoom.tapir;

import freemarker.template.Configuration;
import freemarker.template.Template;
import freemarker.template.TemplateException;
import freemarker.template.TemplateExceptionHandler;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.StandardOpenOption;
import java.util.HashMap;
import java.util.List;
import java.util.Objects;

public class TargetGenerator {
	private final Path outputPath;
	private final Path templatePath;
	private final boolean verbose;
	public static String ENDPOINTS_TEMPLATE_NAME = "endpoints.ftl";
	private final List<EndpointNode> endpoints;

	public static class TargetGeneratorException extends Exception {
		public TargetGeneratorException(String message) {
			super(message);
		}
		public TargetGeneratorException(Exception cause) {
			super(cause);
		}
	}

	public TargetGenerator(
		final boolean verbose,
		Path outputPath,
		Path templatePath,
		List<EndpointNode> endpoints
	) {
		this.verbose = verbose;
		this.outputPath = Objects.requireNonNull(
			outputPath,
			"Output path is required"
		);
		this.templatePath = Objects.requireNonNull(
			templatePath,
			"Template path is required"
		);
		this.endpoints = Objects.requireNonNull(endpoints) ;
	}

	public void generate() throws TargetGeneratorException, IOException, TemplateException {
		Configuration cfg = new Configuration(Configuration.VERSION_2_3_34);
		cfg.setDirectoryForTemplateLoading(this.templatePath.toFile());
		cfg.setDefaultEncoding("UTF-8");
		cfg.setTemplateExceptionHandler(TemplateExceptionHandler.RETHROW_HANDLER);
		cfg.setLogTemplateExceptions(false);
		cfg.setWrapUncheckedExceptions(true);
		cfg.setFallbackOnNullLoopVariable(false);
		Template temp = cfg.getTemplate(ENDPOINTS_TEMPLATE_NAME);
		try (var outputFile = Files.newBufferedWriter(
			outputPath.resolve("endpoints.scala"),
			StandardOpenOption.CREATE,
			StandardOpenOption.TRUNCATE_EXISTING
		)) {
			HashMap<String, List<EndpointNode>> templateData = new HashMap<>();
			templateData.put("endpoints", endpoints);
			temp.process(templateData, outputFile);
		}
	}
}
